#include "types.h"
#include "x86.h"
#include "defs.h"
#include "date.h"
#include "param.h"
#include "memlayout.h"
#include "mmu.h"
#include "proc.h"

int
sys_fork(void)
{
  return fork();
}

int
sys_exit(void)
{
  exit();
  return 0;  // not reached
}

int
sys_wait(void)
{
  return wait();
}

int
sys_kill(void)
{
  int pid;

  if(argint(0, &pid) < 0)
    return -1;
  return kill(pid);
}

int
sys_getpid(void)
{
  return myproc()->pid;
}

int
sys_sbrk(void)
{
  int addr;
  int n;

  if(argint(0, &n) < 0)
    return -1;
  addr = myproc()->sz;
  if(growproc(n) < 0)
    return -1;
  return addr;
}

/************************ LabQuiz2 ************************/

/*
int
sys_shm_get(void)
{
  int addr;
  int key;

  //  Obtain arguments
  //  determine user virtual page address for mapping shared page
  //    if current address is not page-aligned, skip to a page-aligned
  //    address

  if(shm_growproc(key) < 0)
    return -1;

  return addr;
}
*/

/*
int
sys_getps(void)
{
    //  Obtain arguments 
    // return getps(ps);
}
*/

/************************ LabQuiz2 ************************/

int 
sys_shm_stat(void)
{
  //  call function to print statistics as part of system call.
  return shm_stat();
}


int
sys_sleep(void)
{
  int n;
  uint ticks0;

  if(argint(0, &n) < 0)
    return -1;
  acquire(&tickslock);
  ticks0 = ticks;
  while(ticks - ticks0 < n){
    if(myproc()->killed){
      release(&tickslock);
      return -1;
    }
    sleep(&ticks, &tickslock);
  }
  release(&tickslock);
  return 0;
}

// return how many clock tick interrupts have occurred
// since start.
int
sys_uptime(void)
{
  uint xticks;

  acquire(&tickslock);
  xticks = ticks;
  release(&tickslock);
  return xticks;
}


